import { QueryBase } from '@mobicoop/ddd-library';
import { AlgorithmType } from '../../types/algorithm.types';
import { Waypoint } from '../../types/waypoint.type';
import { Frequency } from '@modules/ad/core/domain/ad.types';
import { MatchRequestDto } from '@modules/ad/interface/grpc-controllers/dtos/match.request.dto';

export class MatchQuery extends QueryBase {
  driver?: boolean;
  passenger?: boolean;
  readonly frequency: Frequency;
  readonly fromDate: string;
  readonly toDate: string;
  schedule: ScheduleItem[];
  seatsProposed?: number;
  seatsRequested?: number;
  strict?: boolean;
  readonly waypoints: Waypoint[];
  algorithmType?: AlgorithmType;
  remoteness?: number;
  useProportion?: boolean;
  proportion?: number;
  useAzimuth?: boolean;
  azimuthMargin?: number;
  maxDetourDistanceRatio?: number;
  maxDetourDurationRatio?: number;
  readonly page?: number;
  readonly perPage?: number;

  constructor(props: MatchRequestDto) {
    super();
    this.driver = props.driver;
    this.passenger = props.passenger;
    this.frequency = props.frequency;
    this.fromDate = props.fromDate;
    this.toDate = props.toDate;
    this.schedule = props.schedule;
    this.seatsProposed = props.seatsProposed;
    this.seatsRequested = props.seatsRequested;
    this.strict = props.strict;
    this.waypoints = props.waypoints;
    this.algorithmType = props.algorithmType;
    this.remoteness = props.remoteness;
    this.useProportion = props.useProportion;
    this.proportion = props.proportion;
    this.useAzimuth = props.useAzimuth;
    this.azimuthMargin = props.azimuthMargin;
    this.maxDetourDistanceRatio = props.maxDetourDistanceRatio;
    this.maxDetourDurationRatio = props.maxDetourDurationRatio;
    this.page = props.page ?? 1;
    this.perPage = props.perPage ?? 10;
  }

  setMissingMarginDurations = (defaultMarginDuration: number): MatchQuery => {
    this.schedule.forEach((day: ScheduleItem) => {
      if (day.margin === undefined) day.margin = defaultMarginDuration;
    });
    return this;
  };

  setMissingStrict = (strict: boolean): MatchQuery => {
    if (this.strict === undefined) this.strict = strict;
    return this;
  };

  setDefaultDriverAndPassengerParameters = (
    defaultDriverAndPassengerParameters: DefaultDriverAndPassengerParameters,
  ): MatchQuery => {
    this.driver = !!this.driver;
    this.passenger = !!this.passenger;
    if (!this.driver && !this.passenger) {
      this.driver = defaultDriverAndPassengerParameters.driver;
      this.seatsProposed = defaultDriverAndPassengerParameters.seatsProposed;
      this.passenger = defaultDriverAndPassengerParameters.passenger;
      this.seatsRequested = defaultDriverAndPassengerParameters.seatsRequested;
      return this;
    }
    if (!this.seatsProposed || this.seatsProposed <= 0)
      this.seatsProposed = defaultDriverAndPassengerParameters.seatsProposed;
    if (!this.seatsRequested || this.seatsRequested <= 0)
      this.seatsRequested = defaultDriverAndPassengerParameters.seatsRequested;
    return this;
  };

  setDefaultAlgorithmParameters = (
    defaultAlgorithmParameters: DefaultAlgorithmParameters,
  ): MatchQuery => {
    if (!this.algorithmType)
      this.algorithmType = defaultAlgorithmParameters.algorithmType;
    if (!this.remoteness)
      this.remoteness = defaultAlgorithmParameters.remoteness;
    if (this.useProportion == undefined)
      this.useProportion = defaultAlgorithmParameters.useProportion;
    if (!this.proportion)
      this.proportion = defaultAlgorithmParameters.proportion;
    if (this.useAzimuth == undefined)
      this.useAzimuth = defaultAlgorithmParameters.useAzimuth;
    if (!this.azimuthMargin)
      this.azimuthMargin = defaultAlgorithmParameters.azimuthMargin;
    if (!this.maxDetourDistanceRatio)
      this.maxDetourDistanceRatio =
        defaultAlgorithmParameters.maxDetourDistanceRatio;
    if (!this.maxDetourDurationRatio)
      this.maxDetourDurationRatio =
        defaultAlgorithmParameters.maxDetourDurationRatio;
    return this;
  };
}

type ScheduleItem = {
  day?: number;
  time: string;
  margin?: number;
};

interface DefaultDriverAndPassengerParameters {
  driver: boolean;
  passenger: boolean;
  seatsProposed: number;
  seatsRequested: number;
}

interface DefaultAlgorithmParameters {
  algorithmType: AlgorithmType;
  remoteness: number;
  useProportion: boolean;
  proportion: number;
  useAzimuth: boolean;
  azimuthMargin: number;
  maxDetourDistanceRatio: number;
  maxDetourDurationRatio: number;
}
